@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_GROUP_PERMLIB_H
-#define POLYMAKE_GROUP_PERMLIB_H
+#pragma once
 #undef DOMAIN
 #include "polymake/Array.h"
 #include "polymake/ListMatrix.h"
@@ -32,10 +31,12 @@
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wconversion"
 #pragma clang diagnostic ignored "-Wzero-as-null-pointer-constant"
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
 #elif defined(__GNUC__)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wconversion"
 #pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #endif
 
 #include "permlib/permlib_api.h"
@@ -68,8 +69,37 @@ public:
    PermlibGroup(){};
    PermlibGroup(boost::shared_ptr<permlib::PermutationGroup> perm_group) : permlib_group(perm_group){};
 
-   PermlibGroup(const Array<Array<Int>>& generators)
+   PermlibGroup(const Array<Array<Int>>& _generators)
    {
+      /*
+        Issue #1194:
+
+        For a non-empty list of generators, we translate them directly into permlib format 
+        and return a pointer to a permlib::PermutationGroup object.
+        This also works if the list of generators consists of only an identity permutation.
+
+        However, if the list of generators is empty, paradoxically we have to do more work, because
+        permlib does not deal gracefully with a trivial group specified by giving no generators.
+        One issue that arises is that the degree (=cardinality of the ground set) of such a permutation is not known;
+        another issue is that the implementation of the Schreier-Sims algorithm fails in this case.
+
+        We work around this by substituting an empty list of generators 
+        by the list consisting of the permutation (0), of degree 1.
+
+        This has the disadvantage that the degree of the permutation group given by no generators
+        is manually set to 1. 
+        
+        If this should turn out to be a problem (like it did for empty matrices, which turned out to 
+        desperately want to know their number of rows and columns), the code needs to be reworked.
+       */
+      Array<Array<Int>> trivial_group_generators;
+      if (is_zero(_generators.size())) {
+         trivial_group_generators.append({{0}});
+      }
+      const Array<Array<Int>>& generators( is_zero(_generators.size())
+                                           ? trivial_group_generators
+                                           : _generators );
+      
       std::list<boost::shared_ptr<permlib::Permutation>> permutations;
       for (const auto& perm : generators) {
          boost::shared_ptr<permlib::Permutation> gen(new permlib::Permutation(perm.begin(), perm.end()));
@@ -85,7 +115,7 @@ public:
 
    Int order() const
    {
-      return permlib_group->order();
+      return permlib_group->order<Int>();
    }
   
    boost::shared_ptr<permlib::PermutationGroup> 
@@ -228,7 +258,7 @@ public:
    SetType lex_min_representative(const SetType& dGamma_in) const
    {
       permlib::dset dGamma(permlib_group->n);
-      for (const auto& s : dGamma_in)
+      for (const auto s : dGamma_in)
          dGamma.set(s);
       permlib::dset image = smallestSetImage(*permlib_group, dGamma);
       SetType res(permlib_group->n);
@@ -396,7 +426,6 @@ struct iterator_cross_const_helper<std::__tree_const_iterator<_Tp, _NodePtr, _Di
 #endif
 }
 
-#endif // POLYMAKE_PERMLIB_H
 
 // Local Variables:
 // mode:C++
